﻿Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.IO

Public Class MultiThumbnailGenerator
    Implements IDisposable
    ' common variables 
    Private Shared imageDimension As Integer = 200
    Private Shared maxMultiplier As Single = 0.7F
    Private Shared maxDimension As Single = CSng(imageDimension) * maxMultiplier
    Private Shared thinLineWidth As Single = 0.6F
    Private Shared thickLineWidth As Single = 2.0F
    Private Shared maxAngle As Single = 25.0F

    ' instance variables 
    Private random As New Random
    Private surfaceBitmap As New Bitmap(imageDimension, imageDimension)
    Private surfaceGraphics As Graphics
    Private surfaceBounds? As RectangleF

    Public Sub New()
        surfaceGraphics = Graphics.FromImage(surfaceBitmap)
        surfaceGraphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic
        surfaceGraphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality
    End Sub

    Public Sub AddImage(ByVal image As Image)
        ' scale image
        Dim widthMultiplier = maxDimension / CSng(image.Width)
        Dim heightMultiplier = maxDimension / CSng(image.Height)
        Dim minMultiplier = Math.Min(Math.Min(widthMultiplier, heightMultiplier), 1.0F)
        Dim newWidth = minMultiplier * CSng(image.Width)
        Dim newHeight = minMultiplier * CSng(image.Height)

        ' choose a random translation & rotation
        Dim angle = GetRandomAngle()
        Dim newDimensions = RotateAndFindNewDimensions(New PointF(newWidth, newHeight), angle)
        Dim offset = GetRandomOffset(newDimensions)
        surfaceGraphics.TranslateTransform(offset.X, offset.Y)
        surfaceGraphics.RotateTransform(angle)

        ' draw borders + image
        DrawRectangle(Brushes.White, newWidth, newHeight, thickLineWidth)
        DrawRectangle(Brushes.Black, newWidth, newHeight, thinLineWidth)
        surfaceGraphics.DrawImage(image, -newWidth / 2.0F, -newHeight / 2.0F, newWidth, newHeight)

        ' calculate new image boundaries
        Dim newBounds As New RectangleF(offset.X - newDimensions.X / 2.0F,
                                        offset.Y - newDimensions.Y / 2.0F,
                                        newDimensions.X,
                                        newDimensions.Y)

        If surfaceBounds.HasValue Then
            surfaceBounds = Union(surfaceBounds.Value, newBounds)
        Else
            surfaceBounds = newBounds
        End If

        surfaceGraphics.ResetTransform()
    End Sub

    Private Sub DrawRectangle(ByVal brush As Brush,
                              ByVal width As Single,
                              ByVal height As Single,
                              ByVal lineWidth As Single)
        surfaceGraphics.FillRectangle(brush,
                                      -width / 2.0F - lineWidth,
                                      -height / 2.0F - lineWidth,
                                      width + 2.0F * lineWidth,
                                      height + 2.0F * lineWidth)
    End Sub

    Private Shared Function RotateAndFindNewDimensions(ByVal point As PointF,
                                                       ByVal angle As Single
                                                       ) As PointF
        Using matrix As New Matrix
            Dim points() = {New PointF(0.0F, 0.0F),
                                      New PointF(0.0F, point.Y),
                                      New PointF(point.X, 0.0F),
                                      New PointF(point.X, point.Y)}

            matrix.Rotate(angle)
            matrix.TransformPoints(points)
            Dim minX = points(0).X
            Dim maxX = minX
            Dim minY = points(0).Y
            Dim maxY = minY
            For i = 1 To 3
                minX = Math.Min(minX, points(i).X)
                maxX = Math.Max(maxX, points(i).X)
                minY = Math.Min(minY, points(i).Y)
                maxY = Math.Max(maxY, points(i).Y)
            Next i

            Return New PointF(maxX - minX, maxY - minY)
        End Using
    End Function

    Private Shared Function Union(ByVal oldBounds As RectangleF,
                                  ByVal newBounds As RectangleF
                                  ) As RectangleF
        Dim left = Math.Min(oldBounds.Left, newBounds.Left)
        Dim top = Math.Min(oldBounds.Top, newBounds.Top)
        Dim right = Math.Max(oldBounds.Right, newBounds.Right)
        Dim bottom = Math.Max(oldBounds.Bottom, newBounds.Bottom)
        Return New RectangleF(left, top, right - left, bottom - top)
    End Function

    Private Function GetRandomAngle() As Single
        Dim r = CSng(random.NextDouble())
        Return (r * 2.0F - 1.0F) * maxAngle
    End Function

    Private Function GetRandomOffset(ByVal dimensions As PointF) As PointF
        Dim deltaX = CSng(imageDimension) - dimensions.X - 2.0F * (thinLineWidth + 1.0F)
        Dim deltaY = CSng(imageDimension) - dimensions.Y - 2.0F * (thinLineWidth + 1.0F)

        Dim rX = CSng(random.NextDouble())
        Dim rY = CSng(random.NextDouble())

        Dim newX = (deltaX * rX) + (dimensions.X / 2.0F) + thinLineWidth + 1.0F
        Dim newY = (deltaY * rY) + (dimensions.Y / 2.0F) + thinLineWidth + 1.0F

        Return New PointF(newX, newY)
    End Function

    Public Sub Dispose() Implements IDisposable.Dispose
        surfaceGraphics.Dispose()
        surfaceBitmap.Dispose()
    End Sub

    Public Sub WritePngToStream(ByVal outStream As Stream)
        Using outBitmap As New Bitmap(imageDimension, imageDimension)
            Using outGraphics = Graphics.FromImage(outBitmap)

                outGraphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic
                outGraphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality

                ' center the image
                Dim bounds = If(surfaceBounds, New RectangleF)
                Dim deltaX = (imageDimension - bounds.Width) / 2.0F - bounds.Left
                Dim deltaY = (imageDimension - bounds.Height) / 2.0F - bounds.Top
                outGraphics.DrawImage(surfaceBitmap, New PointF(deltaX, deltaY))

                outBitmap.Save(outStream, System.Drawing.Imaging.ImageFormat.Png)
            End Using
        End Using
    End Sub

End Class