﻿@Imports System.Collections.Specialized
@Imports System.Security
@Imports System.Text
@Imports System.Web.WebPages.Scope
@Functions 
    Private Const FacebookCredentialsTableName As String = "webpages_FacebookCredentials"
    Private Const FacebookCredentialsIdColumn As String = "FacebookId"
    Private Const FacebookCredentialsUserIdColumn As String = "UserId"

    Private Const DefaultUserIdColumn As String = "UserId"
    Private Const DefaultUserNameColumn As String = "email"
    Private Const DefaultUserTableName As String = "UserProfile"

    Private Const DefaultFacebookPerms As String = "email"
    Private Const DefaultCallbackUrl As String = "~/Facebook/Login"
    Private Const FacebookApiProfileUrl As String = "https://graph.facebook.com/me"

    Private Shared ReadOnly _isInitializedKey As New Object
    Private Shared ReadOnly _membershipDBNameKey As New Object
    Private Shared ReadOnly _appIdKey As New Object
    Private Shared ReadOnly _appSecretKey As New Object
    Private Shared ReadOnly _language As New Object

    Public Shared ReadOnly Property HasMembershipIntegration As Boolean
        Get
            Return Not MembershipDBName.IsEmpty()
        End Get
    End Property

    Public Shared ReadOnly Property IsFacebookUserAuthenticated As Boolean
        Get
            Return Not GetFacebookCookieInfo("uid").IsEmpty()
        End Get
    End Property

    Public Shared ReadOnly Property IsFacebookUserAssociated As Boolean
        Get
            Return Not GetAssociatedMembershipUserName().IsEmpty()
        End Get
    End Property

    Public Shared Property IsInitialized As Boolean
        Get
            Return CBool(If(ScopeStorage.CurrentScope(_isInitializedKey), False))
        End Get

        Private Set(ByVal value As Boolean)
            ScopeStorage.CurrentScope(_isInitializedKey) = value
        End Set
    End Property

    Public Shared Property MembershipDBName As String
        Get
            Return CStr(If(ScopeStorage.CurrentScope(_membershipDBNameKey), ""))
        End Get

        Set(ByVal value As String)
            ScopeStorage.CurrentScope(_membershipDBNameKey) = value
        End Set
    End Property

    Public Shared Property AppId As String
        Get
            Return CStr(If(ScopeStorage.CurrentScope(_appIdKey), ""))
        End Get

        Set(ByVal value As String)
            ScopeStorage.CurrentScope(_appIdKey) = value
        End Set
    End Property

    Public Shared Property AppSecret As String
        Get
            Return CStr(If(ScopeStorage.CurrentScope(_appSecretKey), ""))
        End Get

        Set(ByVal value As String)
            ScopeStorage.CurrentScope(_appSecretKey) = value
        End Set
    End Property

    Public Shared Property Language As String
        Get
            Return CStr(If(ScopeStorage.CurrentScope(_language), "en_US"))
        End Get

        Set(ByVal value As String)
            ScopeStorage.CurrentScope(_language) = value
        End Set
    End Property

    '''<summary>
    ''' Initialize the helper with your Facebook application settings. 
    ''' 
    ''' If the 'membershipDBName' parameter is specified, Facebook membership integration will be enabled, 
    ''' allowing users to register and associate their Facebook user account (identified with the e-mail) 
    ''' with your site membership and the WebSecurity helper. 
    ''' In this case, the helper will initialize the WebSecurity WebMatrix helper automatically (if not done previously) 
    ''' and the store the Facebook membership information in the 'membershipDbName' database.    
    '''</summary>
    '''<param name="appId">Facebook application id.</param>        
    '''<param name="appSecret">Facebook application secret.</param>   
    '''<param name="membershipDBName">Name of the database used for storing the membership data.</param>   
    Public Shared Sub Initialize(ByVal appId As String,
                                 ByVal appSecret As String,
                                 Optional ByVal membershipDbName As String = "")
        appId = appId
        appSecret = appSecret
        IsInitialized = True

        If Not membershipDbName.IsEmpty() Then
            membershipDbName = membershipDbName

            InitializeMembershipProviderIfNeeded()
            InitializeFacebookTableIfNeeded()
        End If
    End Sub

    '''<summary>
    ''' Retrieves the Facebook profile of current logged in user.
    '''</summary>
    Public Shared Function GetFacebookUserProfile() As UserProfile
        Dim accessToken = GetFacebookCookieInfo("access_token")

        If accessToken.IsEmpty() Then
            Return Nothing
        End If

        Dim userProfileUrl = New Uri(String.Format("{0}?access_token={1}", FacebookApiProfileUrl, accessToken))

        Dim receiveStream = New WebClient.OpenRead(userProfileUrl)
        Dim result = New StreamReader(receiveStream).ReadToEnd()
        Dim profile = Json.Decode(Of UserProfile)(result)

        Return profile
    End Function

    '''<summary>
    ''' Associates the specified User Name (e.g. email, depending on your membership model) with the current Facebook User Id from the logged user.
    '''</summary>
    '''<param name="userId">The user name to associate the current logged-in facebook account to.</param>
    Public Shared Sub AssociateMembershipAccount(ByVal userName As String)
        If Not IsFacebookUserAuthenticated Then
            Throw New InvalidOperationException("No Facebook user is authenticated.")
        End If

        If IsFacebookUserAssociated Then
            Throw New InvalidOperationException("The authenticated Facebook user is already associated to a membership account.")
        End If

        Using db As Database = Database.Open(MembershipDBName)
            Dim facebookUserId = GetFacebookCookieInfo("uid").As(Of Long)()

            Dim userId = WebSecurity.GetUserId(userName)
            db.Execute(String.Format("INSERT INTO {0} ({1}, {2}) VALUES (@0, @1)",
                                     FacebookCredentialsTableName,
                                     FacebookCredentialsUserIdColumn,
                                     FacebookCredentialsIdColumn),
                                     userId,
                                     facebookUserId)

            ' User is registered in the application
            FormsAuthentication.SetAuthCookie(userName, False)
        End Using
    End Sub

    Public Shared Function MembershipLogin() As Boolean
        Dim user = GetAssociatedMembershipUserName()

        If Not user.IsEmpty() Then
            ' User is registered in the application
            FormsAuthentication.SetAuthCookie(user, False)
            Return True
        Else
            Return False
        End If
    End Function

    Private Shared Sub InitializeMembershipProviderIfNeeded()
        Dim provider = GetMembershipProvider()

        If IsMembershipProviderInitialized(provider) Then
            WebSecurity.InitializeDatabaseConnection(MembershipDBName,
                                                     DefaultUserTableName,
                                                     DefaultUserIdColumn,
                                                     DefaultUserNameColumn, True)
        End If
    End Sub

    Private Shared Sub InitializeFacebookTableIfNeeded()
        Dim provider = GetMembershipProvider()
        Dim db = Database.Open(MembershipDBName)
        Dim table = db.QuerySingle("SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = @0", FacebookCredentialsTableName)

        If table Is Nothing Then
            db.Execute(String.Format("CREATE TABLE {0} ({1} INT NOT NULL, {2} BIGINT NOT NULL)",
                                     FacebookCredentialsTableName,
                                     FacebookCredentialsUserIdColumn,
                                     FacebookCredentialsIdColumn))
        End If
    End Sub

    Private Shared Function GetAssociatedMembershipUserName() As String
        Dim userName = ""

        If IsFacebookUserAuthenticated Then
            Dim db = Database.Open(MembershipDBName)
            Dim provider = GetMembershipProvider()

            Dim userId = db.QueryValue(String.Format("SELECT {0} FROM {1} WHERE {2} = LOWER(@0)",
                                                     FacebookCredentialsUserIdColumn,
                                                     FacebookCredentialsTableName,
                                                     FacebookCredentialsIdColumn),
                                                 GetFacebookCookieInfo("uid"))
            If userId IsNot Nothing Then
                userName = GetUserName(userId)
            End If
        End If

        Return userName
    End Function

    Private Shared Function GetUserName(ByVal userId As Integer) As String
        Dim userName = ""

        Dim db = Database.Open(MembershipDBName)
        Dim provider = GetMembershipProvider()
        userName = db.QueryValue(String.Format("SELECT {0} FROM {1} WHERE {2} = @0",
                                               provider.UserNameColumn,
                                               provider.UserTableName,
                                               provider.UserIdColumn),
                                           userId)

        Return userName
    End Function

    Private Shared Function GetMembershipProvider() As SimpleMembershipProvider
        Dim provider = TryCast(Membership.Provider, SimpleMembershipProvider)

        If provider Is Nothing Then
            Throw New InvalidOperationException("Simple Membership Provider not found.")
        End If

        Return provider
    End Function

    Private Shared Function IsMembershipProviderInitialized(ByVal provider As SimpleMembershipProvider) As Boolean
        Return provider.UserTableName.IsEmpty() OrElse provider.UserIdColumn.IsEmpty() OrElse provider.UserNameColumn.IsEmpty()
    End Function

    Private Shared Function GetFacebookCookieInfo(ByVal key As String) As String
        Dim request = HttpContext.Current.Request
        Dim name = String.Format("fbs_{0}", AppId)

        If request.Cookies(name) IsNot Nothing Then
            Dim value = request.Cookies(name).Value
            Dim args = HttpUtility.ParseQueryString(value.Replace("""", ""))

            If Not IsFacebookCookieValid(args) Then
                Throw New SecurityException("Invalid Facebook cookie.")
            End If

            Return args(key)
        Else
            Return ""
        End If
    End Function

    Private Shared Function IsFacebookCookieValid(ByVal args As NameValueCollection) As Boolean
        Dim payload = New StringBuilder
        For Each key In args.AllKeys
            If key IsNot "sig" Then
                payload.AppendFormat("{0}={1}", key, args(key))
            End If
        Next key

        payload.Append(AppSecret)

        Dim md5 = System.Security.Cryptography.MD5CryptoServiceProvider.Create()
        Dim hash = md5.ComputeHash(Encoding.ASCII.GetBytes(payload.ToString()))
        Dim signature = New StringBuilder
        For i As Integer = 0 To hash.Length - 1
            signature.Append(hash(i).ToString("X2"))
        Next i

        Return String.Equals(args("sig"), signature.ToString(), StringComparison.InvariantCultureIgnoreCase)
    End Function

    Public Class UserProfile
        Public Property Id As String
        Public Property Name As String
        Public Property First_Name As String
        Public Property Last_Name As String
        Public Property Link As String
        Public Property Bio As String
        Public Property Gender As String
        Public Property Email As String
        Public Property Timezone As String
        Public Property Locale As String
        Public Property Updated_Time As String
    End Class
End Functions
@*
Summary: 
  Initialize the Facebook JavaScript SDK to be able to support the XFBML tags of the social plugins.
*@
@Helper GetInitializationScripts()
    @<div id="fb-root">
    </div>
    @<script type="text/javascript">
         window.fbAsyncInit = function () {
             FB.init({ appId: '@AppId', status: true, cookie: true, xfbml: true });
         };
         (function () {
             var e = document.createElement('script'); e.async = true;
             e.src = document.location.protocol +
            '//connect.facebook.net/@(Language)/all.js';
             document.getElementById('fb-root').appendChild(e);
         } ());

         function loginRedirect(url) { window.location = url; }
    </script>
End Helper
@*
Summary:
  Shows a Facebook Login Button, with site membership integration, allowing users to login on your site with their Facebook account (e-mail).
  To use this method, you need to provide the 'membershipDbName' in the helper's Initialize() method.

Parameter: registerUrl()
    Specifies the URL to register the logged user, and associate it with a Membership account.
Parameter: returnUrl()
    Specifies URL to redirect after the user has successfully logged in (i.e. the Facebook User has a Membership Account).
Parameter: callbackUrl()
    Specifies the URL of the WebMatrix page that will handle the Membership login (default URL is ~/Facebook/Login.cshtml).
Parameter: buttonText()
    Specifies the Login Button Text.
Parameter: autoLogoutLink()
    When set to true, if the user is logged into Facebook the button will display a logout button instead.
Parameter: size()
    Specifies the button size: "small", "medium", "large", "xlarge".
Paramter: length()
    Specifies the text lenght: "long" --> 'Connect with Facebook" or "short" --> 'Connect'.
Parameter: showFaces()
    Specifies whether to display profile photos below the button.
Parameter: extendedPermissions()
    The extendedPermissions parameter can be used to request extended permissions from the user. For example, if you want to incorporate a user's photos into your application, set this value to “user_photos”.
*@
@Helper LoginButton(
ByVal registerUrl As String,
Optional ByVal returnUrl As String = "~/",
 Optional ByVal callbackUrl As String = DefaultCallbackUrl,
 Optional ByVal buttonText As String = "",
 Optional ByVal autoLogoutLink As Boolean = False,
 Optional ByVal size As String = "medium",
 Optional ByVal length As String = "long",
 Optional ByVal showFaces As Boolean = False,
 Optional ByVal extendedPermissions As String = "")

     Dim redirectUrl = Href(String.Format("{0}?registerUrl={1}&returnUrl={2}",
                                          callbackUrl,
                                          HttpUtility.UrlEncode(registerUrl),
                                          HttpUtility.UrlEncode(returnUrl)))
     Dim onLogin = String.Format("loginRedirect('{0}')", redirectUrl)
     extendedPermissions = If(extendedPermissions.IsEmpty(), "email", String.Concat("email,", extendedPermissions))

    @<fb:login-button autologoutlink="@autoLogoutLink" size="@size" length="@length" onlogin="@onLogin" show-faces="@showFaces" perms="@extendedPermissions">@buttonText</fb:login-button>        
End Helper
@*
Summary:
  Shows a Facebook Login Button, without integrating Facebook login with your site membership.

Parameter: buttonText
    Specifies the Login Button Text.
Parameter: autoLogoutLink()
    When set to true, if the user is logged into Facebook the button will display a logout button instead.
Parameter: size
    The button size: "small", "medium", "large", "xlarge".
Parameter: length
    The text lenght: "long" --> 'Connect with Facebook" or "short" --> 'Connect'
Parameter: onLogin()
    Specifies the JavaScript action to execute after the user login. 
Parameter: showFaces()
    Whether to display profile photos below the button.
Parameter: extendedPermissions
    The extendedPermissions parameter can be used to request extended permissions from the user. For example, if you want to incorporate a user's photos into your application, set this value to “user_photos”.
*@
@Helper LoginButtonTagOnly(
        Optional ByVal buttonText As String = "",
         Optional ByVal autoLogoutLink As Boolean = False,
         Optional ByVal size As String = "long",
         Optional ByVal length As String = "short",
         Optional ByVal onLogin As String = "",
         Optional ByVal showFaces As Boolean = False,
         Optional ByVal extendedPermissions As String = "")
    @<fb:login-button autologoutlink="@autoLogoutLink" size="@size" length="@length" onlogin="@onLogin" show-faces="@showFaces" perms="@extendedPermissions">@buttonText</fb:login-button>
End Helper
@*
Summary:
  Shows a Facebook Like Button. When the user clicks the Like button on your site, a story appears in the user's friends' News Feed with a link back to your website.

Parameter:   href()
    The URL to like.
Parameter:   buttonLayout()
    The button layout:
        - standard: Displays social text to the right of the button and friends' profile photos below. 
            Minimum width: 225 pixels. Default width: 450 pixels. Height: 35 pixels (without photos) or 80 pixels (with photos). 
        - button_count: Displays the total number of likes to the right of the button. 
            Minimum width: 90 pixels. Default width: 90 pixels. Height: 20 pixels.
Parameter:   showFaces()
    Specifies whether to display profile photos below the button (standard layout only).
Parameter:   width()
    The width of the Like button.
Parameter:   height()
    The height of the plugin in pixels.
Parameter:   action()
    The verb to display on the button: 'like', 'recommend'. 
Parameter:   font()
    The font to display in the button: 'arial', 'lucida grande', 'segoe ui', 'tahoma', 'trebuchet ms', 'verdana'.
Parameter:   colorScheme()
    The color scheme for the like button: 'light' or 'dark'.
Parameter:   refLabel()
    A label for tracking referrals; must be less than 50 characters and can contain alphanumeric characters and some punctuation (currently +/=-.:_).
    
*@
@Helper LikeButton(
        Optional ByVal href As String = "",
        Optional ByVal buttonLayout As String = "standard",
        Optional ByVal showFaces As Boolean = True,
        Optional ByVal width As Integer = 450,
        Optional ByVal height As Integer = 80,
        Optional ByVal action As String = "like",
        Optional ByVal font As String = "",
        Optional ByVal colorScheme As String = "light",
        Optional ByVal refLabel As String = "")
            If href.IsEmpty() Then
                href = Request.Url.ToString()
            End If
        
    @<iframe src="http://www.facebook.com/plugins/like.php?href=@href&layout=@buttonLayout&show_faces=@showFaces&width=@width&action=@action&colorscheme=@colorScheme&height=@height&font=@font&locale=@Language&ref=@refLabel" scrolling="no" frameborder="0" style="border:none; overflow:hidden; width:@(width)px; height:@(height)px;" allowTransparency="true"></iframe>
End Helper
@*
Summary:
    Shows a Facebook Comments plugin.
    The Comments Box easily enables your users to comment on your site's content — whether it's for a web page, article, photo, or other piece of content. 
    Then the user can share the comment on Facebook on their Wall and in their friends' streams.
            An() 'Administer Comments' link will appear below the 'Post' button for developers of the application.

Parameter:  xid()
    An id associated with the comments object (defaults to URL-encoded page URL).
Parameter:  width()
    The width of the plugin, in pixels.
Parameter:  numPosts()
    The number of comments to display, or 0 to hide all comments. 
Parameter:  reverse()
    Changes the order of comments and comment area to allow greater customization.
Parameter:  removeRoundedBox()
    Removes the rounded box around the text area where comments are written to allow greater customization.
*@
@Helper Comments(
Optional ByVal xid As String = "",
Optional ByVal width As Integer = 550,
Optional ByVal numPosts As Integer = 10,
Optional ByVal reverseOrder As Boolean = False,
 Optional ByVal removeRoundedBox As Boolean = False)
    @<fb:comments @(If xid.IsNotEmpty() Then @<text>xid="@xid"</text> End If) numposts="@numPosts" width="@width" reverse="@reverseOrder" simple="@removeRoundedBox" ></fb:comments>
End Helper

@*
Summary:
    Shows a Facebook Recommendations plugin. The Recommendations plugin shows personalized recommendations to your users.
    Since the content is hosted by Facebook, the plugin can display personalized recommendations whether or not the user has logged into your site.
    To generate the recommendations, the plugin considers all the social interactions with URLs from your site.
    For a logged in Facebook user, the plugin will give preference to and highlight objects her friends have interacted with.
    
    *NOTE*: This helper method requires that your site is published into a public address where others can use it. Check this tutorial on publishing with WebMatrix: http://www.asp.net/webmatrix/tutorials/publish-a-website 

Parameter: site
    The address of your published site. For example "www.fourthcoffee.com"
Parameter: width
    The width of the plugin in pixels.
Parameter: height
    The height of the plugin in pixels.
Parameter: showHeader
    Whether to show a 'Recommendations' title bar.
Parameter: colorScheme
    The color scheme of the plugin: 'light' or 'dark'.
Parameter: font
    The font name for the plugin.
Parameter: borderColor
    The border color of the plugin. Use common color names as Red, White, Black, and so on.
Parameter: filter
    Allows you to filter which URLs are shown in the plugin. The plugin will only include URLs which contain the filter term in the first two path parameters of the URL.
Parameter: refLabel
    A label for tracking referrals; must be less than 50 characters and can contain alphanumeric characters and some punctuation (currently +/=-.:_).
*@
    @Helper Recommendations(
        Optional ByVal site As String = "",
        Optional ByVal width As Integer = 300,
        Optional ByVal height As Integer = 300,
        Optional ByVal showHeader As Boolean = True,
         Optional ByVal colorScheme As String = "light",
         Optional ByVal font As String = "",
         Optional ByVal borderColor As String = "",
         Optional ByVal filter As String = "",
         Optional ByVal refLabel As String = "")
             If site.IsEmpty() Then
                 site = Request.Url.Host
             End If
        
        @<iframe src="http://www.facebook.com/plugins/recommendations.php?site=@site&width=@width&height=@height&header=@showHeader&colorscheme=@colorScheme&font=@font&border_color=@borderColor&filter=@filter&ref=@refLabel&locale=@Language" scrolling="no" frameborder="0" style="border:none; overflow:hidden; width:@(width)px; height:@(height)px;" allowTransparency="true"></iframe>
    End Helper

@*
Summary:
    Shows a Facebook Like Box. The Like Box is a social plugin that enables Facebook Page owners to attract and gain Likes from their own website.
    The Like Box enables users to:
		- See how many users already like this page, and which of their friends like it too
		- Read recent posts from the page
		- Like the page with one click, without needing to visit the page   

Parameter: href
    The URL of the Facebook Page for this Like box.
Parameter: width
    The width of the plugin in pixels.
Parameter: height
    The height of the plugin in pixels.
Parameter: colorScheme
    The color scheme of the plugin: 'light' or 'dark'.
Parameter: connections
    Number of shown users who have liked this Page. Use 0 to avoid showing the users box.
Parameter: showStream
    Shows the profile stream for the public profile of the page. 
Parameter: showHeader
    Shows the 'Find us on Facebook' bar at top. Only shown when either stream or connections are present.
*@
    @Helper LikeBox(
ByVal href As String,
Optional ByVal width As Integer = 292,
Optional ByVal height As Integer = 587,
Optional ByVal colorScheme As String = "light",
Optional ByVal connections As Integer = 10,
Optional ByVal showStream As Boolean = True,
Optional ByVal showHeader As Boolean = True)       
        @<iframe src="http://www.facebook.com/plugins/likebox.php?href=@href&width=@width&height=@height&colorscheme=@colorScheme&connections=@connections&stream=@showStream&header=@showHeader&locale=@Language" scrolling="no" frameborder="0" style="border:none; overflow:hidden; width:@(width)px; height:@(height)px;" allowTransparency="true"></iframe>                
    End Helper

@*
Summary:
    Shows a Facebook Facepile plugin.
    The Facepile plugin shows the Facebook profile pictures of the user's friends who have already signed up for your site. 
    The plugin doesn't show up if the user is logged out of Facebook or doesn't have friends who have signed up for your site using Facebook.

Parameter: maxRows
    The maximum number of rows of profile pictures to show. The plugin dynamically sizes its height; for example, if you specify a maximum of four rows, and there are only enough friends to fill two rows, the height of the plugin will be only what is needed for two rows of profile pictures. 
Parameter: width
    The width of the plugin in pixels. 
*@
    @Helper Facepile(
Optional ByVal maxRows As Integer = 1,
 Optional ByVal width As Integer = 200)
        @<fb:facepile max-rows="@maxRows" width="@width"></fb:facepile>
    End Helper

@*
Summary:
    Shows a Facebook Live Stream plugin.
    The Live Stream plugin lets users visiting your site or application share activity and comments in real time. 
    The Live Stream Box works best when you are running a real-time event, like live streaming video for concerts, speeches, or webcasts, live Web chats, 
    webinars, massively multiplayer games.

Parameter: width
    The width of the plugin in pixels.
Parameter: height
    The height of the plugin in pixels.
Parameter: xid
    If you have multiple live stream boxes on the same page, specify a unique 'xid' for each.
Parameter: viaUrl
    The URL that users are redirected to when they click on your app name on a status (if not specified, your Connect URL is used).
Parameter: alwaysPostToFriends
    If set, all user posts will always go to their profile. This option should only be used when users' posts are likely to make sense outside of the context of the event.
*@
    @Helper LiveStream(
Optional ByVal width As Integer = 400,
Optional ByVal height As Integer = 500,
Optional ByVal xid As String = "",
Optional ByVal viaUrl As String = "",
Optional ByVal allwaysPostToFriends As Boolean = False)
        @<iframe src="http://www.facebook.com/plugins/live_stream_box.php?app_id=@AppId&width=@width&height=@height@(If Not xid.IsEmpty() Then @<text>&xid=@xid</text> End If)@(If Not xid.IsEmpty() Then @<text>&via_url=@viaUrl</text> End If)&always_post_to_friends=@allwaysPostToFriends&locale=@Language" scrolling="no" frameborder="0" style="border:none; overflow:hidden; width:@(width)px; height:@(height)px;" allowTransparency="true"></iframe>
    End Helper

@*
Summary:
    Shows a Facebook Activity Feed plugin.    
    The Activity Feed plugin displays the most interesting recent activity taking place on your site. Since the content is hosted by Facebook, 
    the plugin can display personalized content whether or not the user has logged into your site. The activity feed displays stories both when 
    users like content on your site and when users share content from your site back to Facebook.
    
    *NOTE*: This helper method requires that your site is published into a public address where others can use it. Check this tutorial on publishing with WebMatrix: http://www.asp.net/webmatrix/tutorials/publish-a-website 

Parameter: site
    The address of your published site. For example "www.fourthcoffee.com"
Parameter: width
    The width of the plugin in pixels.
Parameter: height
    The height of the plugin in pixels.
Parameter: showHeader
    Show the 'Recent Activity' title bar.
Parameter: colorScheme
    The color scheme of the plugin: 'light' or 'dark'.
Parameter: font
    The font name for the plugin.
Parameter: borderColor
    The border color of the plugin. Use common color names as Red, White, Black, and so on.
Parameter: showRecommendations
    Whether to show recommendations on the activity feed.
*@
    @Helper ActivityFeed(
        Optional ByVal site As String = "",
        Optional ByVal width As Integer = 300,
        Optional ByVal height As Integer = 300,
        Optional ByVal showHeader As Boolean = True,
        Optional ByVal colorScheme As String = "light",
        Optional ByVal font As String = "",
        Optional ByVal borderColor As String = "",
        Optional ByVal showRecommendations As Boolean = False)
    If site.IsEmpty() Then
        site = Request.Url.Host
    End If
                
        @<iframe src="http://www.facebook.com/plugins/activity.php?site=@site&width=@width&height=@height&header=@showHeader&colorscheme=@colorScheme&font=@font&border_color=@borderColor&recommendations=@showRecommendations&locale=@Language" scrolling="no" frameborder="0" style="border:none; overflow:hidden; width:300px; height:300px;" allowTransparency="true"></iframe>
    End Helper

@*
Summary:
    OpenGraph properties allows you to specify structured information about your web pages to show up your pages richly across Facebook and enable Facebook users to establish connections to your pages.
    
    Use this method to show OpenGraph page data, as the page title, URL, and so on.
    
    *NOTE*: It is important to notice that the OpenGraph properties are read directly from your site by Facebook, therefore your site needs to be published into a public address to see the OpenGraph properties working. 
    Check this tutorial on publishing with WebMatrix: http://www.asp.net/webmatrix/tutorials/publish-a-website 

Parameter: siteName
    A human-readable name for your site, e.g., "Fourth Coffee".
Parameter: title
    The title of your page as it should appear within Facebook, e.g., "Pecan Pie".
Parameter: type
    The type of your page, e.g., "food". See the complete list of supported types here: http://developers.facebook.com/docs/opengraph#types
Parameter: url
    The canonical URL of your object that will be used as its permanent ID in the graph, e.g., http://www.fourthcoffe.com/Product/1/.
Parameter: imageUrl
    An image URL which should represent your page within the graph. The image must be at least 50px by 50px and have a maximum aspect ratio of 3:1.
Parameter: description
    A one to two sentence description of your page.
*@
    @Helper OpenGraphRequiredProperties(
ByVal siteName As String,
ByVal title As String,
ByVal type As String,
ByVal url As String,
ByVal imageUrl As String,
Optional ByVal description As String = "")
        @<meta property="og:site_name" content="@siteName"/>
        @<meta property="fb:app_id" content="@AppId"/>     
        @<meta property="og:title" content="@title"/>
        @<meta property="og:type" content="@type"/>
        @<meta property="og:url" content="@url"/>
        @<meta property="og:image" content="@imageUrl"/>        
    If Not description.IsEmpty() Then
        @<meta property="og:description" content="@description"/>
    End If
    End Helper

@*
Summary:
    OpenGraph properties allows you to specify structured information about your web pages to show up your pages richly across Facebook and enable Facebook users to establish connections to your pages.
    
    Use this method to show page location data. This is useful if your pages is a business profile or about anything else with a real-world location. You can specify location via latitude and longitude, a full address, or both.
    
    *NOTE*: It is important to notice that the OpenGraph properties are read directly from your site by Facebook, therefore your site needs to be published into a public address to see the OpenGraph properties working. 
    Check this tutorial on publishing with WebMatrix: http://www.asp.net/webmatrix/tutorials/publish-a-website 
*@     
    @Helper OpenGraphLocationProperties(
            Optional ByVal latitude As String = "",
            Optional ByVal longitude As String = "",
            Optional ByVal streetAddress As String = "",
            Optional ByVal locality As String = "",
            Optional ByVal region As String = "",
            Optional ByVal postalCode As String = "",
            Optional ByVal countryName As String = "")   
    
                If Not latitude.IsEmpty() Then
        @<meta property="og:latitude" content="@latitude"/>
                End If
                If Not longitude.IsEmpty() Then
        @<meta property="og:longitude" content="@longitude"/>
                End If
                If Not streetAddress.IsEmpty() Then
        @<meta property="og:street-address" content="@streetAddress"/>
                End If
                If Not locality.IsEmpty() Then
        @<meta property="og:locality" content="@locality"/>
                End If
                If Not region.IsEmpty() Then
        @<meta property="og:region" content="@region"/>
                End If
                If Not postalCode.IsEmpty() Then
        @<meta property="og:postal-code" content="@postalCode"/>
                End If
                If Not countryName.IsEmpty() Then
        @<meta property="og:country-name" content="@countryName"/>
                End If
    End Helper

@*
Summary:
    OpenGraph properties allows you to specify structured information about your web pages to show up your pages richly across Facebook and enable Facebook users to establish connections to your pages.
    
    Use this method to show contact information about your page. Consider including contact information if your page is about an entity that can be contacted.
    
    *NOTE*: It is important to notice that the OpenGraph properties are read directly from your site by Facebook, therefore your site needs to be published into a public address to see the OpenGraph properties working. 
    Check this tutorial on publishing with WebMatrix: http://www.asp.net/webmatrix/tutorials/publish-a-website 
*@
    @Helper OpenGraphContactProperties(
        Optional ByVal email As String = "",
        Optional ByVal phoneNumber As String = "",
        Optional ByVal faxNumber As String = "")    
    
            If Not email.IsEmpty() Then
        @<meta property="og:email" content="@email"/>
            End If
            If Not phoneNumber.IsEmpty() Then
        @<meta property="og:phone_number" content="@phoneNumber"/>
            End If
            If Not faxNumber.IsEmpty() Then
        @<meta property="og:fax_number" content="@faxNumber"/>
            End If
    
    End Helper

@*
Summary:
    Use this method inside your opening HTML tag for W3C compatibility.
        For example: <html xmlns="http://www.w3.org/1999/xhtml" @Facebook.FbmlNamespaces()>         
*@
    @Helper FbmlNamespaces() 
        @<text>xmlns:fb="http://www.facebook.com/2008/fbml" xmlns:og="http://opengraphprotocol.org/schema/"</text> 
    End Helper
